=== modified file 'drizzled/db.cc'
--- drizzled/db.cc	2010-10-24 01:34:39 +0000
+++ drizzled/db.cc	2010-10-26 17:07:58 +0000
@@ -272,7 +272,7 @@
     else
     {
       LOCK_open.lock(); /* After deleting database, remove all cache entries related to schema */
-      remove_db_from_cache(schema_identifier);
+      table::Cache::singleton().removeSchema(schema_identifier);
       LOCK_open.unlock();
 
 
@@ -399,9 +399,9 @@
     {
       Table *locked_table;
       abort_locked_tables(session, identifier);
-      remove_table_from_cache(session, identifier,
-                              RTFC_WAIT_OTHER_THREAD_FLAG |
-                              RTFC_CHECK_KILLED_FLAG);
+      table::Cache::singleton().removeTable(session, identifier,
+                                            RTFC_WAIT_OTHER_THREAD_FLAG |
+                                            RTFC_CHECK_KILLED_FLAG);
       /*
         If the table was used in lock tables, remember it so that
         unlock_table_names can free it

=== modified file 'drizzled/lock.cc'
--- drizzled/lock.cc	2010-10-26 16:07:45 +0000
+++ drizzled/lock.cc	2010-10-26 17:07:58 +0000
@@ -718,7 +718,7 @@
   table_list->table= reinterpret_cast<Table *>(table);
 
   /* Return 1 if table is in use */
-  return(test(remove_table_from_cache(session, identifier, RTFC_NO_FLAG)));
+  return(test(table::Cache::singleton().removeTable(session, identifier, RTFC_NO_FLAG)));
 }
 
 

=== modified file 'drizzled/sql_base.cc'
--- drizzled/sql_base.cc	2010-10-26 16:07:45 +0000
+++ drizzled/sql_base.cc	2010-10-26 17:07:58 +0000
@@ -256,7 +256,7 @@
       for (TableList *table= tables; table; table= table->next_local)
       {
         TableIdentifier identifier(table->getSchemaName(), table->getTableName());
-        if (remove_table_from_cache(session, identifier,
+        if (table::Cache::singleton().removeTable(session, identifier,
                                     RTFC_OWNED_BY_Session_FLAG))
         {
           found= true;
@@ -3982,165 +3982,7 @@
   unireg support functions
  *****************************************************************************/
 
-/*
-  Invalidate any cache entries that are for some DB
-
-  SYNOPSIS
-  remove_db_from_cache()
-  db		Database name. This will be in lower case if
-  lower_case_table_name is set
-
-NOTE:
-We can't use hash_delete when looping hash_elements. We mark them first
-and afterwards delete those marked unused.
-*/
-
-void remove_db_from_cache(const SchemaIdentifier &schema_identifier)
-{
-  safe_mutex_assert_owner(LOCK_open.native_handle());
-
-  for (table::CacheMap::const_iterator iter= table::getCache().begin();
-       iter != table::getCache().end();
-       iter++)
-  {
-    table::Concurrent *table= (*iter).second;
-
-    if (not schema_identifier.getPath().compare(table->getShare()->getSchemaName()))
-    {
-      table->getMutableShare()->resetVersion();			/* Free when thread is ready */
-      if (not table->in_use)
-        table::getUnused().relink(table);
-    }
-  }
-
-  table::getUnused().cullByVersion();
-}
-
-
-/*
-  Mark all entries with the table as deleted to force an reopen of the table
-
-  The table will be closed (not stored in cache) by the current thread when
-  close_thread_tables() is called.
-
-  PREREQUISITES
-  Lock on LOCK_open()
-
-  RETURN
-  0  This thread now have exclusive access to this table and no other thread
-  can access the table until close_thread_tables() is called.
-  1  Table is in use by another thread
-*/
-
-bool remove_table_from_cache(Session *session, TableIdentifier &identifier, uint32_t flags)
-{
-  const TableIdentifier::Key &key(identifier.getKey());
-  bool result= false; 
-  bool signalled= false;
-
-  for (;;)
-  {
-    result= signalled= false;
-
-    table::CacheRange ppp;
-    ppp= table::getCache().equal_range(key);
-
-    for (table::CacheMap::const_iterator iter= ppp.first;
-         iter != ppp.second; ++iter)
-    {
-      table::Concurrent *table= (*iter).second;
-      Session *in_use;
-
-      table->getMutableShare()->resetVersion();		/* Free when thread is ready */
-      if (not (in_use= table->in_use))
-      {
-        table::getUnused().relink(table);
-      }
-      else if (in_use != session)
-      {
-        /*
-          Mark that table is going to be deleted from cache. This will
-          force threads that are in mysql_lock_tables() (but not yet
-          in thr_multi_lock()) to abort it's locks, close all tables and retry
-        */
-        in_use->some_tables_deleted= true;
-        if (table->is_name_opened())
-        {
-          result= true;
-        }
-        /*
-          Now we must abort all tables locks used by this thread
-          as the thread may be waiting to get a lock for another table.
-          Note that we need to hold LOCK_open while going through the
-          list. So that the other thread cannot change it. The other
-          thread must also hold LOCK_open whenever changing the
-          open_tables list. Aborting the MERGE lock after a child was
-          closed and before the parent is closed would be fatal.
-        */
-        for (Table *session_table= in_use->open_tables;
-             session_table ;
-             session_table= session_table->getNext())
-        {
-          /* Do not handle locks of MERGE children. */
-          if (session_table->db_stat)	// If table is open
-            signalled|= mysql_lock_abort_for_thread(session, session_table);
-        }
-      }
-      else
-      {
-        result= result || (flags & RTFC_OWNED_BY_Session_FLAG);
-      }
-    }
-
-    table::getUnused().cullByVersion();
-
-    /* Remove table from table definition cache if it's not in use */
-    TableShare::release(identifier);
-
-    if (result && (flags & RTFC_WAIT_OTHER_THREAD_FLAG))
-    {
-      /*
-        Signal any thread waiting for tables to be freed to
-        reopen their tables
-      */
-      broadcast_refresh();
-      if (!(flags & RTFC_CHECK_KILLED_FLAG) || !session->killed)
-      {
-        dropping_tables++;
-        if (likely(signalled))
-        {
-          boost_unique_lock_t scoped(LOCK_open, boost::adopt_lock_t());
-          COND_refresh.wait(scoped);
-          scoped.release();
-        }
-        else
-        {
-          /*
-            It can happen that another thread has opened the
-            table but has not yet locked any table at all. Since
-            it can be locked waiting for a table that our thread
-            has done LOCK Table x WRITE on previously, we need to
-            ensure that the thread actually hears our signal
-            before we go to sleep. Thus we wait for a short time
-            and then we retry another loop in the
-            remove_table_from_cache routine.
-          */
-          boost::xtime xt; 
-          xtime_get(&xt, boost::TIME_UTC); 
-          xt.sec += 10; 
-          boost_unique_lock_t scoped(LOCK_open, boost::adopt_lock_t());
-          COND_refresh.timed_wait(scoped, xt);
-          scoped.release();
-        }
-        dropping_tables--;
-        continue;
-      }
-    }
-    break;
-  }
-
-  return result;
-}
+
 
 
 /**

=== modified file 'drizzled/sql_table.cc'
--- drizzled/sql_table.cc	2010-10-24 19:36:55 +0000
+++ drizzled/sql_table.cc	2010-10-26 17:07:58 +0000
@@ -180,9 +180,9 @@
     {
       Table *locked_table;
       abort_locked_tables(session, tmp_identifier);
-      remove_table_from_cache(session, tmp_identifier,
-                              RTFC_WAIT_OTHER_THREAD_FLAG |
-                              RTFC_CHECK_KILLED_FLAG);
+      table::Cache::singleton().removeTable(session, tmp_identifier,
+                                            RTFC_WAIT_OTHER_THREAD_FLAG |
+                                            RTFC_CHECK_KILLED_FLAG);
       /*
         If the table was used in lock tables, remember it so that
         unlock_table_names can free it
@@ -1680,7 +1680,7 @@
 
   /* Wait until all there are no other threads that has this table open */
   TableIdentifier identifier(table->getShare()->getSchemaName(), table->getShare()->getTableName());
-  remove_table_from_cache(session, identifier, RTFC_WAIT_OTHER_THREAD_FLAG);
+  table::Cache::singleton().removeTable(session, identifier, RTFC_WAIT_OTHER_THREAD_FLAG);
 }
 
 /*
@@ -1935,7 +1935,7 @@
         {
           boost::unique_lock<boost::mutex> lock(LOCK_open);
 	  TableIdentifier identifier(table->table->getShare()->getSchemaName(), table->table->getShare()->getTableName());
-          remove_table_from_cache(session, identifier, RTFC_NO_FLAG);
+          table::Cache::singleton().removeTable(session, identifier, RTFC_NO_FLAG);
         }
       }
     }

=== modified file 'drizzled/table/cache.cc'
--- drizzled/table/cache.cc	2010-10-26 16:07:45 +0000
+++ drizzled/table/cache.cc	2010-10-26 17:07:58 +0000
@@ -27,13 +27,19 @@
 
 #include "drizzled/identifier/table.h"
 #include "drizzled/table.h"
+#include "drizzled/session.h"
 #include "drizzled/table/concurrent.h"
 
 #include "drizzled/table/cache.h"
 #include "drizzled/table/unused.h"
 
+#include "drizzled/pthread_globals.h"
+
 namespace drizzled
 {
+
+class Session;
+
 namespace table
 {
 
@@ -117,5 +123,164 @@
   return 0;
 }
 
+/*
+  Invalidate any cache entries that are for some DB
+
+  SYNOPSIS
+  remove_db_from_cache()
+  db		Database name. This will be in lower case if
+  lower_case_table_name is set
+
+NOTE:
+We can't use hash_delete when looping hash_elements. We mark them first
+and afterwards delete those marked unused.
+*/
+
+void Cache::removeSchema(const SchemaIdentifier &schema_identifier)
+{
+  //safe_mutex_assert_owner(LOCK_open.native_handle());
+
+  for (table::CacheMap::const_iterator iter= table::getCache().begin();
+       iter != table::getCache().end();
+       iter++)
+  {
+    table::Concurrent *table= (*iter).second;
+
+    if (not schema_identifier.getPath().compare(table->getShare()->getSchemaName()))
+    {
+      table->getMutableShare()->resetVersion();			/* Free when thread is ready */
+      if (not table->in_use)
+        table::getUnused().relink(table);
+    }
+  }
+
+  table::getUnused().cullByVersion();
+}
+
+/*
+  Mark all entries with the table as deleted to force an reopen of the table
+
+  The table will be closed (not stored in cache) by the current thread when
+  close_thread_tables() is called.
+
+  PREREQUISITES
+  Lock on LOCK_open()
+
+  RETURN
+  0  This thread now have exclusive access to this table and no other thread
+  can access the table until close_thread_tables() is called.
+  1  Table is in use by another thread
+*/
+
+bool Cache::removeTable(Session *session, TableIdentifier &identifier, uint32_t flags)
+{
+  const TableIdentifier::Key &key(identifier.getKey());
+  bool result= false; 
+  bool signalled= false;
+
+  for (;;)
+  {
+    result= signalled= false;
+
+    table::CacheRange ppp;
+    ppp= table::getCache().equal_range(key);
+
+    for (table::CacheMap::const_iterator iter= ppp.first;
+         iter != ppp.second; ++iter)
+    {
+      table::Concurrent *table= (*iter).second;
+      Session *in_use;
+
+      table->getMutableShare()->resetVersion();		/* Free when thread is ready */
+      if (not (in_use= table->in_use))
+      {
+        table::getUnused().relink(table);
+      }
+      else if (in_use != session)
+      {
+        /*
+          Mark that table is going to be deleted from cache. This will
+          force threads that are in mysql_lock_tables() (but not yet
+          in thr_multi_lock()) to abort it's locks, close all tables and retry
+        */
+        in_use->some_tables_deleted= true;
+        if (table->is_name_opened())
+        {
+          result= true;
+        }
+        /*
+          Now we must abort all tables locks used by this thread
+          as the thread may be waiting to get a lock for another table.
+          Note that we need to hold LOCK_open while going through the
+          list. So that the other thread cannot change it. The other
+          thread must also hold LOCK_open whenever changing the
+          open_tables list. Aborting the MERGE lock after a child was
+          closed and before the parent is closed would be fatal.
+        */
+        for (Table *session_table= in_use->open_tables;
+             session_table ;
+             session_table= session_table->getNext())
+        {
+          /* Do not handle locks of MERGE children. */
+          if (session_table->db_stat)	// If table is open
+            signalled|= mysql_lock_abort_for_thread(session, session_table);
+        }
+      }
+      else
+      {
+        result= result || (flags & RTFC_OWNED_BY_Session_FLAG);
+      }
+    }
+
+    table::getUnused().cullByVersion();
+
+    /* Remove table from table definition cache if it's not in use */
+    TableShare::release(identifier);
+
+    if (result && (flags & RTFC_WAIT_OTHER_THREAD_FLAG))
+    {
+      /*
+        Signal any thread waiting for tables to be freed to
+        reopen their tables
+      */
+      broadcast_refresh();
+      if (!(flags & RTFC_CHECK_KILLED_FLAG) || !session->killed)
+      {
+        dropping_tables++;
+        if (likely(signalled))
+        {
+          boost_unique_lock_t scoped(LOCK_open, boost::adopt_lock_t());
+          COND_refresh.wait(scoped);
+          scoped.release();
+        }
+        else
+        {
+          /*
+            It can happen that another thread has opened the
+            table but has not yet locked any table at all. Since
+            it can be locked waiting for a table that our thread
+            has done LOCK Table x WRITE on previously, we need to
+            ensure that the thread actually hears our signal
+            before we go to sleep. Thus we wait for a short time
+            and then we retry another loop in the
+            remove_table_from_cache routine.
+          */
+          boost::xtime xt; 
+          xtime_get(&xt, boost::TIME_UTC); 
+          xt.sec += 10; 
+          boost_unique_lock_t scoped(LOCK_open, boost::adopt_lock_t());
+          COND_refresh.timed_wait(scoped, xt);
+          scoped.release();
+        }
+        dropping_tables--;
+        continue;
+      }
+    }
+    break;
+  }
+
+  return result;
+}
+
 } /* namespace table */
 } /* namespace drizzled */

=== modified file 'drizzled/table/cache.h'
--- drizzled/table/cache.h	2010-10-26 16:07:45 +0000
+++ drizzled/table/cache.h	2010-10-26 17:07:58 +0000
@@ -50,6 +50,8 @@
   }
 
   bool areTablesUsed(Table *table, bool wait_for_name_lock);
+  void removeSchema(const SchemaIdentifier &schema_identifier);
+  bool removeTable(Session *session, TableIdentifier &identifier, uint32_t flags);
 };
 
 CacheMap &getCache(void);

